@@ -18,12 +18,15 @@
 (define (cpp-err fmt . args)
   (apply throw 'cpp-error fmt args))
 
-;;.@deffn skip-ws ch
-(define (skip-ws ch)
-  (if (eof-object? ch) ch
-      (if (char-set-contains? c:ws ch)
-	  (skip-ws (read-char))
-	  ch)))
+;;.@deffn skip-il-ws ch
+;; Skip in-line whitespace
+(define skip-il-ws
+  (let ((il-ws (list->char-set '(#\space #\tab))))
+    (lambda (ch)
+      (cond
+       ((eof-object? ch) ch)
+       ((char-set-contains? il-ws ch) (skip-il-ws (read-char)))
+       (else ch)))))
 
 ;; Since we want to be able to get CPP statements with comment in tact
 ;; (e.g., for passing to @code{pretty-print-c99}) we need to remove
@@ -98,14 +101,28 @@
 	    (else (error "incomplete implementation"))))))
     (eval-expr tree)))
 
-;; @deffn scan-cpp-input argd used dict for-argl => string
-;; Process the replacement text and generate a (reversed) token-list.
-;; If for-argl, stop at, and push back, @code{,} or @code{)}.
-(define (scan-cpp-input argd dict used for-argl)
+;; Note: scan-cpp-input scans replacement text.  When identifiers are found
+;; they are tested for expansion as follows:
+;; @enumerate
+;; @item If already expanded, then ignore.
+;; @item If followed by @code{(}, then use @code{collect-args} to get the
+;; arguments and ...
+;; @item Otherwise insert the replacement text and continue scanning (at
+;; first character of new replacement text.
+;; @end enumerate
+
+;; @deffn scan-cpp-input argd used dict end-tok => string
+;; Process replacement text from the input port and generate a (reversed)
+;; token-list.  If end-tok, stop at, and push back, @code{,} or @code{)}.
+;; If end-tok is @code{,} then read until @code{,} or @code{(}.
+;; The argument @var{argd} is a dictionary (argument name, argument
+;; value) pairs which will be expanded as needed.  This routine is called
+;; by collect-args, expand-cpp-repl and cpp-expand-text.
+(define (scan-cpp-input argd dict used end-tok)
   ;; Works like this: scan tokens (comments, parens, strings, char's, etc).
   ;; Tokens (i.e., strings) are collected in a (reverse ordered) list (stl)
   ;; and merged together on return.  Lone characters are collected in the
-  ;; list @code{chl}.  Once a non-char token is found the character list is
+  ;; list @code{chl}.  Once a token border is seen the character list is
   ;; converted to a string and added to the string list first, followed by
   ;; the new token.
 
@@ -113,22 +130,35 @@
   (define (add-chl chl stl)
     (if (null? chl) stl (cons (list->string (reverse chl)) stl)))
 
+  ;; used when we see `#foo'; converts foo to "foo"
+  (define (stringify str)
+    (string-append "\"" str "\""))
+
+  (define conjoin string-append)
+
   ;; We just scanned "defined", now need to scan the arg to inhibit expansion.
   ;; For example, we have scanned "defined"; we now scan "(FOO)" or "FOO", and
-  ;; return "defined(FOO)".  We use ec (end-char) as state indicator: nul at
-  ;; start, #\) on seeing #\( or #\space if other.
-  (define (scan-defined)
-    (let* ((ch (skip-ws (read-char))) (ec (if (char=? ch #\() #\) #\space)))
-      (let iter ((chl '(#\()) (ec ec) (ch ch))
+  ;; return "defined(FOO)".  We use ec (end-char) as terminal char:
+  ;; #\) if starts with #( or #\nul if other.
+  (define (scan-defined-arg)
+    (let* ((ch (skip-il-ws (read-char)))
+	   (ec (if (char=? ch #\() #\) #\null)))
+      (let iter ((chl '(#\()) (ec ec) (ch (if (char=? ec #\)) (read-char) ch)))
 	(cond
-	 ((and (eof-object? ch) (char=? #\space ec))
-	  (string-append "defined" (list->string (reverse (cons #\) chl)))))
-	 ((eof-object? ch) (cpp-err "illegal argument to `defined'"))
-	 ((and (char=? ch #\)) (char=? ec #\)))
-	  (string-append "defined" (list->string (reverse (cons ch chl)))))
+	 ((eof-object? ch)
+	  (if (char=? ec #\null)
+	      (string-append "defined" (list->string (reverse (cons #\) chl))))
+	      (cpp-err "illegal argument to `defined'")))
 	 ((char-set-contains? c:ir ch)
 	  (iter (cons ch chl) ec (read-char)))
-	 (else (cpp-err "illegal identifier"))))))
+	 ((char=? ec #\))
+	  (if (char=? #\) (skip-il-ws ch))
+	      (string-append "defined" (list->string (reverse (cons #\) chl))))
+	      (cpp-err "garbage in argument to `defined'")))
+	 ((char=? ec #\null) ;; past identifier
+	  (string-append "defined" (list->string (reverse (cons #\) chl)))))
+	 (else
+	  (cpp-err "illegal argument to  `defined'"))))))
 
   (let iter ((stl '())		; string list (i.e., tokens)
 	     (chl '())		; char-list (current list of input chars)
@@ -141,29 +171,40 @@
      ;; If end of string or see end-ch at level 0, then return.
      ((eof-object? ch)  ;; CHECK (ab++)
       (apply string-append (reverse (add-chl chl stl))))
-     ((and for-argl (memq ch '(#\) #\,)) (zero? lvl))
+     
+     ((and (eqv? end-tok ch) (zero? lvl))
       (unread-char ch) (apply string-append (reverse (add-chl chl stl))))
+     ((and end-tok (char=? #\) ch) (zero? lvl))
+      (unread-char ch) (apply string-append (reverse (add-chl chl stl))))
+     
      ((read-c-comm ch #f) =>
       (lambda (cp) (iter stl chl (string-append "/*" (cdr cp) "*/")
 			 lvl (read-char))))
+     ;; not sure about this:
+     ((char-set-contains? c:ws ch) (iter stl chl nxt lvl (read-char)))
      ((char=? #\( ch) (iter stl (cons ch chl) nxt (1+ lvl) (read-char)))
      ((char=? #\) ch) (iter stl (cons ch chl) nxt (1- lvl) (read-char)))
      ((char=? #\# ch)
       (let ((ch (read-char)))
 	(if (eqv? ch #\#)
-	    (iter (cons "##" stl) chl #f lvl (read-char))
-	    (iter (cons "#" stl) chl #f lvl ch))))
+	    (iter stl chl "##" lvl (read-char))
+	    (iter stl chl "#" lvl ch))))
      ((read-c-string ch) =>
       (lambda (st) (iter stl chl st lvl (read-char))))
      ((read-c-ident ch) =>
       (lambda (iden)
 	(if (equal? iden "defined")
 	    ;; "defined" is a special case
-	    (iter stl chl (scan-defined) lvl (read-char))
+	    (let ((arg (scan-defined-arg)))
+	      (iter stl chl arg lvl (read-char)))
 	    ;; otherwise ...
 	    (let* ((aval (assoc-ref argd iden))  ; lookup argument
 		   (rval (assoc-ref dict iden))) ; lookup macro def
 	      (cond
+	       ((and (pair? stl) (string=? "#" (car stl)))
+		(iter (cdr stl) chl (stringify aval) lvl (read-char)))
+	       ((and (pair? stl) (string=? "##" (car stl)))
+		(iter (cddr stl) chl (conjoin (cadr stl) aval) lvl (read-char)))
 	       ((member iden used)	; name used
 		(iter stl chl iden lvl (read-char)))
 	       (aval			; arg ref
@@ -172,22 +213,39 @@
 		(iter stl chl rval lvl (read-char)))
 	       ((pair? rval)		; cpp macro
 		(let* ((argl (car rval)) (text (cdr rval))
-		       (argv (collect-args argd dict used))
-		       (argd (map cons argl argv))
+		       (argd (collect-args argl argd dict used))
 		       (newl (expand-cpp-repl text argd dict (cons iden used))))
 		  (iter stl chl newl lvl (read-char))))
 	       (else			; normal identifier
 		(iter stl chl iden lvl (read-char))))))))
      (else
       (iter stl (cons ch chl) #f lvl (read-char))))))
-  
-(define (collect-args argd dict used)
-  (if (not (eqv? (skip-ws (read-char)) #\()) (cpp-err "CPP expecting `('"))
-  (let iter ((argl (list (scan-cpp-input argd dict used #t))))
-    (let ((ch (read-char)))
-      (if (eqv? ch #\)) (reverse argl)
-	  (iter (cons (scan-cpp-input argd dict used #t) argl))))))
-    
+
+;; @deffn collect-args argl argd dict used => argd
+;; to be documented
+;; I think argd is a passthrough for scan-cpp-input
+;; argl: list of formal arguments in #define
+;; argd: used? (maybe just a pass-through for scan-cpp-input
+;; dict: dict of macro defs
+;; used: list of already expanded macros
+;; TODO clean this up
+;; should be looking at #\( and eat up to matching #\)
+(define (collect-args argl argd dict used)
+  (let iter ((argl argl) (argv '()) (ch (skip-il-ws (read-char))))
+    ;; ch should always be #\(, #\, or #\)
+    (cond
+     ((eqv? ch #\)) (reverse argv))
+     ((null? argl) (cpp-err "arg count"))
+     ((and (null? (cdr argl)) (string=? (car argl) "..."))
+      (let ((val (scan-cpp-input argd dict used #\))))
+	(iter (cdr argl) (acons "__VA_ARGS__" val argv) (read-char))))
+     ((or (eqv? ch #\() (eqv? ch #\,))
+      (let ((val (scan-cpp-input argd dict used #\,)))
+	(iter (cdr argl) (acons (car argl) val argv) (read-char))))
+     (else (error "coding error, ch=" ch)))))
+
+;; @deffn expand-cpp-repl
+;; to be documented
 (define (expand-cpp-repl repl argd dict used)
   (with-input-from-string repl
     (lambda () (scan-cpp-input argd dict used #f))))
@@ -213,9 +271,8 @@
       (let ((expd (expand-cpp-repl rval '() dict (cons ident used))))
 	expd))
      ((pair? rval)
-      (let* ((args (car rval)) (repl (cdr rval))
-	     (argv (collect-args '() dict '()))
-	     (argd (map cons args argv))
+      (let* ((argl (car rval)) (repl (cdr rval))
+	     (argd (collect-args argl '() dict '()))
 	     (expd (expand-cpp-repl repl argd dict (cons ident used))))
 	expd)))))
 
